@@ -46,7 +46,9 @@ from treeview import NXTreeView
 from plotview import NXPlotView
 from datadialogs import *
 from scripteditor import ScriptDialog
-from nexpy.api.nexus.tree import nxload, NeXusError, NXFile, NXlink, NXobject
+from nexpy.api.nexus.tree import (nxload, NeXusError, NXFile, 
+                                  NXobject, NXfield, NXgroup, NXlink,
+                                  NXroot, NXentry)
 
 # IPython imports
 # require minimum version of IPython for RichIPythonWidget()
@@ -55,7 +57,6 @@ pkg_resources.require("IPython>="+'1.1.0')
 from IPython.qt.console.rich_ipython_widget import RichIPythonWidget
 from IPython.qt.inprocess import QtInProcessKernelManager
 
-ansi_re = re.compile('\x1b' + r'\[([\dA-Fa-f;]*?)m')
 
 def report_error(context, error):
     title = type(error).__name__ + ': ' + context
@@ -100,7 +101,7 @@ class MainWindow(QtGui.QMainWindow):
 
         rightpane = QtGui.QWidget()
 
-        self.plotview = NXPlotView(label="Main",parent=rightpane)
+        self.plotview = NXPlotView(label="Main", parent=rightpane)
         self.plotview.setMinimumSize(700, 550)
 
         self.console = RichIPythonWidget(config=self.config, parent=rightpane)
@@ -163,8 +164,8 @@ class MainWindow(QtGui.QMainWindow):
         self.init_menu_bar()
 
         self.file_filter = ';;'.join((
-             "NeXus Files (*.nxs *.nx5 *.h5 *.hdf *.hdf5)",
-	         "Any Files (*.* *)"))
+            "NeXus Files (*.nxs *.nx5 *.h5 *.hdf *.hdf5)",
+            "Any Files (*.* *)"))
         self.setWindowTitle('NeXpy')
         self.statusBar().showMessage('Ready')
         self.console._control.setFocus()
@@ -910,7 +911,6 @@ class MainWindow(QtGui.QMainWindow):
             report_error("Unlocking File", error)
 
     def show_import_dialog(self):
-        sender = self.sender()
         import_module = self.importer[self.sender()]
         self.import_dialog = import_module.ImportDialog()
         self.import_dialog.show()
@@ -1329,27 +1329,11 @@ class MainWindow(QtGui.QMainWindow):
 
     def show_log(self):
         try:
-            from consoleapp import _nexpy_dir
-            f = open(os.path.join(_nexpy_dir, 'nexpy.log'), 'r')
-            text = ansi_re.sub('', f.read())
-            f.close()
-            self.log_window = QtGui.QDialog(self)
-            layout = QtGui.QVBoxLayout()
-            text_box = QtGui.QPlainTextEdit(text)
-            text_box.setMinimumWidth(700)
-            text_box.setMinimumHeight(600)
-            layout.addWidget(text_box)
-            buttonbox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Close)
-            buttonbox.rejected.connect(self.close_log)
-            layout.addWidget(buttonbox)
-            self.log_window.setLayout(layout)
-            self.log_window.exec_()
+            dialog = LogDialog(self)
+            dialog.show()
         except NeXusError as error:
             report_error("Showing Log File", error)
 
-    def close_log(self):
-        self.log_window.close()
-    
     def new_script(self):
         try:
             file_name = None
